/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: Katherine Don, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 2006 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.cdm;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import org.greenstone.gatherer.collection.CollectionManager;
import org.greenstone.gatherer.Configuration;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.gui.DesignPaneHeader;
import org.greenstone.gatherer.gui.GLIButton;

public class MacrosManager
    extends JPanel {

    /** The controls used to modify the general options. */
    private Control controls;
    /** Constructor. */
    public MacrosManager() {
    }


    /** Destructor. */
    public void destroy() {
	if (controls != null) {
	    controls.destroy();
	    controls = null;
	}
    }

    public void loseFocus() {
    }

    public void gainFocus() {

    }
    /** This class is resposible for generating the controls for the editing of general options.
     * @return the Control for editing the general options
     */
    public Control getControls() {
	if (controls == null) {
	    controls = new MacrosControl();
	}
	return controls;
    }
    

    /** Called when the detail mode has changed which in turn may cause several design elements to be available/hidden
     * @param mode the new mode as an int
     */
    public void modeChanged(int mode) {

    }


    private class MacrosControl
	extends JPanel
	implements Control {

	JTextArea macros_textarea = null;
	JButton save_button = null;
	JButton reset_button = null;
	
	public MacrosControl() {
	    super();

	    JPanel header_pane = new DesignPaneHeader("CDM.GUI.Macros", "collectionspecificmacros");
	    
	    JPanel main_pane = new JPanel();
	    main_pane.setLayout(new BorderLayout());
	    macros_textarea = new JTextArea();
	    macros_textarea.setBackground(Configuration.getColor("coloring.editable_background", false));
	    macros_textarea.setCaretPosition(0);
	    macros_textarea.setLineWrap(false);
	    macros_textarea.setWrapStyleWord(false);
	    Dictionary.registerTooltip(macros_textarea, "CDM.MacrosManager.Editor_Tooltip");

	    JScrollPane macros_scroll = new JScrollPane(macros_textarea);
	    
	    save_button = new GLIButton();
	    Dictionary.registerBoth(save_button, "CDM.MacrosManager.Save", "CDM.MacrosManager.Save_Tooltip");
	    reset_button = new GLIButton();
	    Dictionary.registerBoth(reset_button, "CDM.MacrosManager.Reset", "CDM.MacrosManager.Reset_Tooltip");
	    
	    save_button.addActionListener(new SaveListener());
	    reset_button.addActionListener(new ResetListener());
	    JPanel button_pane = new JPanel();
	    button_pane.setLayout(new GridLayout(1,2));
	    button_pane.add(save_button);
	    button_pane.add(reset_button);

	    main_pane.add(macros_scroll, BorderLayout.CENTER);
	    main_pane.add(button_pane, BorderLayout.SOUTH);

	    setBorder(BorderFactory.createEmptyBorder(0,5,0,0));
	    setLayout(new BorderLayout());
	    add(header_pane, BorderLayout.NORTH);
	    add(main_pane, BorderLayout.CENTER);

	    
	    
	    
	}
	public void destroy() {

	}
	public void loseFocus() {
	}

	public void gainFocus() {
	    readMacroFile();
	}

	private void readMacroFile() {
	    macros_textarea.setText("");
	    File extra_dm = new File(CollectionManager.getCollectionDirectoryPath()+"macros"+File.separator+"extra.dm");
	    
	    if (extra_dm.exists()) {
		try {
		    BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(extra_dm), "UTF-8"));
		
		    String line;
		    while ((line = br.readLine()) != null) {
			macros_textarea.append(line+"\n");
		    }
		} catch (Exception e) {
		    System.err.println("Exception happened, "+e.getMessage());
		}
	    }
	}
	
	private void writeMacroFile() {
	    File extra_dm = new File(CollectionManager.getCollectionDirectoryPath()+"macros"+File.separator+"extra.dm");
	    try {
		if (!extra_dm.exists()) {
		    File parent_dir = extra_dm.getParentFile();
		    parent_dir.mkdirs();
		    extra_dm.createNewFile();
		}
		
		BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(extra_dm), "UTF-8"));
		String text = macros_textarea.getText();
		out.write(text, 0, text.length());
		out.newLine();
		out.flush();
		out.close();
	    } catch (Exception e) {
		
	    }
	}
	private class SaveListener
	    implements ActionListener {

	    public void actionPerformed(ActionEvent event) {
		writeMacroFile();
	    }
	}
	
	private class ResetListener
	    implements ActionListener {

	    public void actionPerformed(ActionEvent event) {
		readMacroFile();
	    }
	}
    }
	
}
